@@ -1560,9 +1560,8 @@ glade_base_editor_class_init (GladeBaseEditorClass *klass)
                     G_TYPE_FROM_CLASS (object_class),
                     G_SIGNAL_RUN_LAST,
                     G_STRUCT_OFFSET (GladeBaseEditorClass, build_child),
-                    _glade_stop_emission_accumulator, NULL,
-                    _glade_marshal_OBJECT__OBJECT_UINT,
-                    G_TYPE_OBJECT, 2, G_TYPE_OBJECT, G_TYPE_UINT);
+                    _glade_stop_emission_accumulator, NULL, NULL,
+                    G_TYPE_OBJECT, 2, G_TYPE_OBJECT, G_TYPE_GTYPE);
 
   /**
    * GladeBaseEditor::delete-child:
@@ -1801,14 +1800,14 @@ glade_base_editor_init (GladeBaseEditor *editor)
   gtk_grid_set_column_spacing (GTK_GRID (button_table), 8);
   gtk_box_pack_start (GTK_BOX (tree_vbox), button_table, FALSE, TRUE, 0);
 
-  button = gtk_button_new_from_stock (GTK_STOCK_ADD);
+  button = gtk_button_new_from_icon_name ("list-add", GTK_ICON_SIZE_BUTTON);
   gtk_widget_set_hexpand (button, TRUE);
   gtk_widget_show (button);
   g_signal_connect (button, "clicked",
                     G_CALLBACK (glade_base_editor_add_activate), editor);
   gtk_grid_attach (GTK_GRID (button_table), button, 0, 0, 1, 1);
 
-  e->remove_button = button = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
+  e->remove_button = button = gtk_button_new_from_icon_name ("list-remove", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (button);
   gtk_widget_set_hexpand (button, TRUE);
   g_signal_connect (button, "clicked",
@@ -2018,7 +2017,8 @@ glade_base_editor_add_default_properties (GladeBaseEditor *editor,
 
   /* Name */
   label = gtk_label_new (_("Name:"));
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.0);
+  gtk_widget_set_halign (label, GTK_ALIGN_END);
+  gtk_widget_set_valign (label, GTK_ALIGN_START);
 
   entry = gtk_entry_new ();
   gtk_entry_set_text (GTK_ENTRY (entry), glade_widget_get_name (gchild));
@@ -2033,7 +2033,8 @@ glade_base_editor_add_default_properties (GladeBaseEditor *editor,
     {
       /* Type */
       label = gtk_label_new (_("Type:"));
-      gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.0);
+      gtk_widget_set_halign (label, GTK_ALIGN_END);
+      gtk_widget_set_valign (label, GTK_ALIGN_START);
 
       entry = gtk_combo_box_new ();
       gtk_combo_box_set_model (GTK_COMBO_BOX (entry), child_class);
@@ -2156,8 +2157,10 @@ glade_base_editor_add_label (GladeBaseEditor *editor, gchar *str)
   row = editor->priv->row;
 
   gtk_label_set_markup (GTK_LABEL (label), markup);
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
-  gtk_misc_set_padding (GTK_MISC (label), 0, 6);
+  gtk_widget_set_halign (label, GTK_ALIGN_START);
+  gtk_widget_set_valign (label, GTK_ALIGN_START);
+  gtk_widget_set_margin_top (label, 6);
+  gtk_widget_set_margin_bottom (label, 6);
 
   gtk_grid_attach (GTK_GRID (editor->priv->table), label, 0, row, 2, 1);
   gtk_widget_show (label);
@@ -2243,7 +2246,7 @@ glade_base_editor_pack_new_window (GladeBaseEditor *editor,
   gtk_box_set_spacing (GTK_BOX (buttonbox), 8);
   gtk_box_pack_start (GTK_BOX (editor), buttonbox, FALSE, TRUE, 0);
 
-  button = gtk_button_new_from_stock (GTK_STOCK_HELP);
+  button = gtk_button_new_from_icon_name ("help-browser", GTK_ICON_SIZE_BUTTON);
   gtk_widget_show (button);
   g_signal_connect (button, "clicked",
                     G_CALLBACK (glade_base_editor_help),
