@@ -41,12 +41,11 @@
 #include "glade-design-layout.h"
 #include "glade-design-private.h"
 #include "glade-path.h"
+#include "glade-adaptor-chooser.h"
 
 #include <glib.h>
 #include <glib/gi18n.h>
 
-#define GLADE_DESIGN_VIEW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), GLADE_TYPE_DESIGN_VIEW, GladeDesignViewPrivate))
-
 #define GLADE_DESIGN_VIEW_KEY "GLADE_DESIGN_VIEW_KEY"
 
 enum
@@ -71,6 +70,7 @@ static GtkVBoxClass *parent_class = NULL;
 static void glade_design_view_drag_init (_GladeDragInterface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (GladeDesignView, glade_design_view, GTK_TYPE_BOX,
+                         G_ADD_PRIVATE (GladeDesignView)
                          G_IMPLEMENT_INTERFACE (GLADE_TYPE_DRAG,
                                                 glade_design_view_drag_init))
 
@@ -328,7 +328,7 @@ logo_draw (GtkWidget *widget, cairo_t *cr)
 static gboolean
 glade_design_view_draw (GtkWidget *widget, cairo_t *cr)
 {
-  GladeDesignViewPrivate *priv = GLADE_DESIGN_VIEW_GET_PRIVATE (widget);
+  GladeDesignViewPrivate *priv = GLADE_DESIGN_VIEW (widget)->priv;
   GdkWindow *window = gtk_widget_get_window (widget);
   gboolean should_draw = gtk_cairo_should_draw_window (cr, window);
   gboolean sw_visible = gtk_widget_get_visible (priv->scrolled_window);
@@ -365,12 +365,52 @@ glade_design_view_draw (GtkWidget *widget, cairo_t *cr)
   return FALSE;
 }
 
+static void
+on_chooser_adaptor_selected (_GladeAdaptorChooser *chooser,
+                             GladeWidgetAdaptor   *adaptor,
+                             GladeProject         *project)
+
+{
+  gtk_widget_hide (GTK_WIDGET (chooser));
+  glade_command_create (adaptor, NULL, NULL, project);
+  gtk_widget_destroy (GTK_WIDGET (chooser));
+}
+
+static gboolean
+glade_design_view_viewport_button_press (GtkWidget       *widget,
+                                         GdkEventButton  *event,
+                                         GladeDesignView *view)
+{
+  GladeDesignViewPrivate *priv = view->priv;
+  GdkRectangle rect = {event->x, event->y, 8, 8};
+  GtkWidget *pop, *chooser;
+  
+  if (event->type != GDK_2BUTTON_PRESS)
+    return FALSE;
+
+  pop = gtk_popover_new (widget);
+  gtk_popover_set_pointing_to (GTK_POPOVER (pop), &rect);
+  gtk_popover_set_position (GTK_POPOVER (pop), GTK_POS_BOTTOM);
+  
+  chooser = _glade_adaptor_chooser_new (GLADE_ADAPTOR_CHOOSER_TOPLEVEL,
+                                        priv->project);
+  g_signal_connect (chooser, "adaptor-selected",
+                    G_CALLBACK (on_chooser_adaptor_selected),
+                    priv->project);
+
+  gtk_container_add (GTK_CONTAINER (pop), chooser);
+  gtk_widget_show (chooser);
+  gtk_widget_show (pop);
+
+  return TRUE;
+}
+
 static void
 glade_design_view_init (GladeDesignView *view)
 {
   GtkWidget *viewport;
 
-  view->priv = GLADE_DESIGN_VIEW_GET_PRIVATE (view);
+  view->priv = glade_design_view_get_instance_private (view);
 
   gtk_widget_set_no_show_all (GTK_WIDGET (view), TRUE);
   gtk_orientable_set_orientation (GTK_ORIENTABLE (view),
@@ -387,6 +427,10 @@ glade_design_view_init (GladeDesignView *view)
                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
   viewport = gtk_viewport_new (NULL, NULL);
+  gtk_widget_add_events (viewport, GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
+  g_signal_connect (viewport, "button-press-event",
+                    G_CALLBACK (glade_design_view_viewport_button_press),
+                    view);
   gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);
   gtk_container_add (GTK_CONTAINER (viewport), view->priv->layout_box);
   gtk_container_add (GTK_CONTAINER (view->priv->scrolled_window), viewport);
@@ -715,8 +759,6 @@ glade_design_view_class_init (GladeDesignViewClass *klass)
                                                         GLADE_TYPE_PROJECT,
                                                         G_PARAM_READWRITE |
                                                         G_PARAM_CONSTRUCT_ONLY));
-
-  g_type_class_add_private (object_class, sizeof (GladeDesignViewPrivate));
 }
 
 /* Private API */
