@@ -747,7 +747,7 @@ glade_editor_query_dialog (GladeWidget *widget)
   dialog = gtk_dialog_new_with_buttons (title, NULL,
                                         GTK_DIALOG_MODAL |
                                         GTK_DIALOG_DESTROY_WITH_PARENT,
-                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                        _("_Cancel"), GTK_RESPONSE_CANCEL,
                                         NULL);
   g_free (title);
 
@@ -1103,8 +1103,8 @@ glade_editor_reset_dialog_run (GtkWidget *parent, GladeWidget *gwidget)
                                         parent ? GTK_WINDOW (parent) : NULL,
                                         GTK_DIALOG_MODAL |
                                         GTK_DIALOG_DESTROY_WITH_PARENT,
-                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                        GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
+                                        _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                        _("_OK"), GTK_RESPONSE_OK, NULL);
 
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
   gtk_widget_show (vbox);
@@ -1118,7 +1118,7 @@ glade_editor_reset_dialog_run (GtkWidget *parent, GladeWidget *gwidget)
   /* Checklist */
   label = gtk_label_new_with_mnemonic (_("_Properties:"));
   gtk_widget_show (label);
-  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+  gtk_widget_set_halign (label, GTK_ALIGN_START);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
 
   sw = gtk_scrolled_window_new (NULL, NULL);
@@ -1165,7 +1165,7 @@ glade_editor_reset_dialog_run (GtkWidget *parent, GladeWidget *gwidget)
   /* Description */
   label = gtk_label_new_with_mnemonic (_("Property _Description:"));
   gtk_widget_show (label);
-  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+  gtk_widget_set_halign (label, GTK_ALIGN_START);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
 
   sw = gtk_scrolled_window_new (NULL, NULL);
@@ -1275,7 +1275,7 @@ glade_editor_dialog_for_widget (GladeWidget *widget)
 
   /* Window */
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_type_hint (GTK_WINDOW (window), GDK_WINDOW_TYPE_HINT_DIALOG);
+  gtk_window_set_type_hint (GTK_WINDOW (window), GDK_WINDOW_TYPE_HINT_UTILITY);
 
   prj_name = glade_project_get_name (glade_widget_get_project (widget));
   /* Translators: first %s is the project name, second is a widget name */
