@@ -732,7 +732,34 @@ glade_gtk_widget_set_property (GladeWidgetAdaptor * adaptor,
   if (!strcmp (id, "can-focus"))
     return;
 
-  GWA_GET_CLASS (G_TYPE_OBJECT)->set_property (adaptor, object, id, value);
+  if (!strcmp (id, "glade-style-classes"))
+    {
+      GtkStyleContext *context = gtk_widget_get_style_context (GTK_WIDGET (object));
+      GList *l, *styles;
+
+      /* NOTE: we can not use gtk_style_context_list_classes() because we only
+       * want to remove the classes we added not the ones added by the widget
+       */
+      styles = g_object_get_data (object, "glade-style-classes");
+      for (l = styles; l; l = g_list_next (l))
+	{
+	  GladeString *style = l->data;
+          gtk_style_context_remove_class (context, style->string);
+	}
+
+      for (l = g_value_get_boxed (value); l; l = g_list_next (l))
+	{
+	  GladeString *style = l->data;
+	  gtk_style_context_add_class (context, style->string);
+	}
+
+      /* Save the list here so we can use it later on to remove them from the style context */
+      g_object_set_data_full (object, "glade-style-classes",
+                              glade_string_list_copy (g_value_get_boxed (value)),
+                              (GDestroyNotify) glade_string_list_free);
+    }
+  else
+      GWA_GET_CLASS (G_TYPE_OBJECT)->set_property (adaptor, object, id, value);
 }
 
 void
@@ -781,14 +808,9 @@ glade_gtk_widget_action_activate (GladeWidgetAdaptor * adaptor,
 {
   GladeWidget *gwidget = glade_widget_get_from_gobject (object), *gparent;
   GList this_widget = { 0, }, that_widget = { 0,};
-  GtkWidget *parent = gtk_widget_get_parent (GTK_WIDGET (object));
   GladeProject *project;
 
-  if (parent)
-    gparent = glade_widget_get_from_gobject (parent);
-  else
-    gparent = NULL;
-
+  gparent = glade_widget_get_parent (gwidget);
   project = glade_widget_get_project (gwidget);
 
   if (strcmp (action_path, "preview") == 0)
@@ -864,31 +886,30 @@ glade_gtk_widget_action_activate (GladeWidgetAdaptor * adaptor,
         {
           GladeWidgetAdaptor *adaptor =
             glade_widget_adaptor_get_by_type (new_type);
+          GList *saved_props, *prop_cmds;
 	  GladeWidget *gnew_parent;
+          GladeProperty *property;
 
           glade_command_push_group (_("Adding parent %s for %s"),
                                     glade_widget_adaptor_get_title (adaptor), 
 				    glade_widget_get_name (gwidget));
 
+          /* Record packing properties */
+          saved_props =
+	    glade_widget_dup_properties (gwidget, glade_widget_get_packing_properties (gwidget),
+					 FALSE, FALSE, FALSE);
+
+	  property = glade_widget_get_parentless_widget_ref (gwidget);
+
+	  /* Remove "this" widget, If the parent we're removing is a parentless 
+	   * widget reference, the reference will be implicitly broken by the 'cut' command */
+          this_widget.data = gwidget;
+          glade_command_delete (&this_widget);
+
           /* Create new widget and put it where the placeholder was */
           if ((gnew_parent =
                glade_command_create (adaptor, gparent, NULL, project)) != NULL)
             {
-              GList *saved_props, *prop_cmds;
-              GladeProperty *property;
-
-              /* Record packing properties */
-              saved_props =
-                glade_widget_dup_properties (gwidget, glade_widget_get_packing_properties (gwidget),
-                                             FALSE, FALSE, FALSE);
-
-              /* Remove "this" widget, If the parent we're removing is a parentless
-               * widget reference, the reference will be implicitly broken by the 'cut' command */
-              this_widget.data = gwidget;
-              glade_command_delete (&this_widget);
-
-              property = glade_widget_get_parentless_widget_ref (gwidget);
-              
 	      /* Now we created the new parent, if gwidget had a parentless widget reference...
 	       * set that reference to the new parent instead */
 	      if (property)
@@ -909,8 +930,6 @@ glade_gtk_widget_action_activate (GladeWidgetAdaptor * adaptor,
               /* Create heavy-duty glade-command properties stuff */
               prop_cmds =
                   create_command_property_list (gnew_parent, saved_props);
-              g_list_foreach (saved_props, (GFunc) g_object_unref, NULL);
-              g_list_free (saved_props);
 
               /* Apply the properties in an undoable way */
               if (prop_cmds)
@@ -919,9 +938,26 @@ glade_gtk_widget_action_activate (GladeWidgetAdaptor * adaptor,
 
               /* Add "this" widget to the new parent */
               glade_command_add (&this_widget, gnew_parent, NULL, project, FALSE);
-            }
 
-          glade_command_pop_group ();
+              glade_command_pop_group ();
+            }
+          else
+	    {
+              glade_command_pop_group ();
+
+              /* Undo delete command
+               * FIXME: this will leave the "Adding parent..." comand in the
+               * redo list, which I think its better than leaving it in the
+               * undo list by using glade_command_add() to add the widget back
+               * to the original parent.
+               * Ideally we need a way to remove a redo item from the project or
+               * simply do not let the user cancel a widget creation!
+               */
+	      glade_project_undo (project);
+	    }
+
+          g_list_foreach (saved_props, (GFunc) g_object_unref, NULL);
+          g_list_free (saved_props);
         }
     }
   else if (strcmp (action_path, "sizegroup_add") == 0)
@@ -930,7 +966,7 @@ glade_gtk_widget_action_activate (GladeWidgetAdaptor * adaptor,
     }
   else if (strcmp (action_path, "clear_properties") == 0)
     {
-      glade_editor_reset_dialog_run (gtk_widget_get_toplevel (parent), gwidget);
+      glade_editor_reset_dialog_run (gtk_widget_get_toplevel (GTK_WIDGET (object)), gwidget);
     }
   else if (strcmp (action_path, "read_documentation") == 0)
     {
