@@ -28,6 +28,16 @@
 
 #define CONFIG_GROUP             "User & Survey"
 
+/* translators: Email subject sent to the user after completing the survey */
+#define MAIL_SUBJECT _("Glade User Survey")
+/* translators: Email body sent to the user after completing the survey */
+#define MAIL_BODY _("Thank you for taking Glade Users survey, we appreciate it!\n\nTo validate this email address open the folowing link\n\nhttps://people.gnome.org/~jpu/glade/registration.php?email=$email&validation_token=$new_validation_token\n\nIn case you want to change or update the survey, your current update token is:\n$new_token\n\nCheers\n\n	The Glade team\n")
+
+/* translators: Email subject sent to the user after updating the survey */
+#define UPDATE_MAIL_SUBJECT _("Glade User Survey (update)")
+/* translators: Email body sent to the user after updating the survey */
+#define UPDATE_MAIL_BODY _("Thank you for updating your Glade Users survey data, we appreciate it!\n\nIn case you want to change something again, your current update token is:\n$new_token\n\nCheers\n\n	The Glade team\n")
+
 struct _GladeRegistrationPrivate
 {
   GtkWidget    *infobar;
@@ -424,16 +434,16 @@ on_http_request_done (GladeHTTP         *http,
                                              "%s", _("Name and Email fields are required"));
           else if (g_strcmp0 (status, "error_email_in_use") == 0)
             glade_registration_show_message (registration, GTK_MESSAGE_WARNING,
-                                             "%s", _("Ops! Email address is already in use!\nTo update information you need to provide the token that was sent to your inbox."));
+                                             "%s", _("Oops! Email address is already in use!\nTo update information you need to provide the token that was sent to your inbox."));
           else if (g_strcmp0 (status, "error_db_user_info") == 0)
             glade_registration_show_message (registration, GTK_MESSAGE_WARNING,
-                                             _("Ops! Error saving user information: %s"), message);
+                                             _("Oops! Error saving user information: %s"), message);
           else if (g_strcmp0 (status, "error_db_survey_data") == 0)
             glade_registration_show_message (registration, GTK_MESSAGE_WARNING,
-                                             _("Ops! Error saving survey data: %s"), message);
+                                             _("Oops! Error saving survey data: %s"), message);
           else if (g_strcmp0 (status, "error_db") == 0)
             glade_registration_show_message (registration, GTK_MESSAGE_WARNING,
-                                             _("Ops! Error accessing DB: %s"), message);
+                                             _("Oops! Error accessing DB: %s"), message);
           else
             glade_registration_show_message (registration, GTK_MESSAGE_WARNING, "%s", message);
         }
